@@ -10,13 +10,15 @@ class TransformCache;
 class TextureCache;
 class material;
 class hitable;
-class bvh_node;
 class random_gen;
 class hitable_list;
+class alpha_texture;
+class bump_texture;
+class roughness_texture;
 
 std::shared_ptr<hitable> rotation_order(std::shared_ptr<hitable> entry, NumericVector temprotvec, NumericVector order_rotation);
 
-std::shared_ptr<bvh_node> build_scene(List& scene,
+std::shared_ptr<hitable> build_scene(List& scene,
                                      IntegerVector& shape,
                                      Float shutteropen, 
                                      Float shutterclose,
@@ -25,12 +27,16 @@ std::shared_ptr<bvh_node> build_scene(List& scene,
                                      std::vector<unsigned char * >& bump_textures, 
                                      std::vector<unsigned char * >& roughness_textures,  
                                      std::vector<std::shared_ptr<material> >* shared_materials, 
+                                     std::vector<std::shared_ptr<alpha_texture> >& alpha,
+                                     std::vector<std::shared_ptr<bump_texture> >& bump,
+                                     std::vector<std::shared_ptr<roughness_texture> >& roughness,
                                      int bvh_type,
                                      TransformCache& transformCache, 
                                      TextureCache& texCache,
                                      hitable_list& imp_sample_objects,
                                      std::vector<std::shared_ptr<hitable> >& instanced_objects,
                                      std::vector<std::shared_ptr<hitable_list> >& instance_importance_sampled,
+                                     std::vector<int>& texture_idx,
                                      bool verbose,
                                      random_gen& rng); 
 
