@@ -1,5 +1,5 @@
 #include "constant.h"
-
+#include "raylog.h"
 
 bool constant_medium::bounding_box(Float t0, Float t1, aabb& box) const {
   return(boundary->bounding_box(t0,t1,box));
@@ -22,7 +22,9 @@ vec3f constant_medium::random(const point3f& o, Sampler* sampler, Float time) {
 }
 
 
-bool constant_medium::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng) {
+const bool constant_medium::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng) const {
+  SCOPED_CONTEXT("Hit");
+  SCOPED_TIMER_COUNTER("Medium");
   hit_record rec1, rec2;
   if(boundary->hit(r, -FLT_MAX, FLT_MAX,rec1, rng)) {
     if(boundary->hit(r, rec1.t + 0.0001, FLT_MAX, rec2, rng)) {
@@ -43,7 +45,7 @@ bool constant_medium::hit(const ray& r, Float t_min, Float t_max, hit_record& re
       if(hit_distance < distance_inside_boundary) {
         rec.t = rec1.t + hit_distance / r.direction().length();
         rec.p = r.point_at_parameter(rec.t);
-        rec.normal = vec3f(1,0,0);
+        rec.normal = normal3f(1,0,0);
         rec.mat_ptr = phase_function.get();
         return(true);
       }
@@ -53,7 +55,9 @@ bool constant_medium::hit(const ray& r, Float t_min, Float t_max, hit_record& re
 }
 
 
-bool constant_medium::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sampler* sampler) {
+const bool constant_medium::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sampler* sampler) const {
+  SCOPED_CONTEXT("Hit");
+  SCOPED_TIMER_COUNTER("Medium");
   hit_record rec1, rec2;
   if(boundary->hit(r, -FLT_MAX, FLT_MAX,rec1, sampler)) {
     if(boundary->hit(r, rec1.t + 0.0001, FLT_MAX, rec2, sampler)) {
@@ -74,7 +78,7 @@ bool constant_medium::hit(const ray& r, Float t_min, Float t_max, hit_record& re
       if(hit_distance < distance_inside_boundary) {
         rec.t = rec1.t + hit_distance / r.direction().length();
         rec.p = r.point_at_parameter(rec.t);
-        rec.normal = vec3f(1,0,0);
+        rec.normal = normal3f(1,0,0);
         rec.mat_ptr = phase_function.get();
         return(true);
       }
