@@ -1,19 +1,21 @@
 #include "hitable.h"
+#include "raylog.h"
+#include <cmath>
 
 //Translate implementation
 
 void get_sphere_uv(const vec3f& p, Float& u, Float& v) {
   Float phi = atan2(p.z(),p.x());
   Float theta = asin(p.y());
-  u = 1 - (phi + M_PI) / (2*M_PI);
-  v = (theta + M_PI/2) / M_PI;
+  u = 1 - (phi * M_1_PI + 1) * 0.5;
+  v = (theta * M_1_PI + 0.5);
 }
 
 void get_sphere_uv(const normal3f& p, Float& u, Float& v) {
   Float phi = atan2(p.z(),p.x());
   Float theta = asin(p.y());
-  u = 1 - (phi + M_PI) / (2*M_PI);
-  v = (theta + M_PI/2) / M_PI;
+  u = 1 - (phi * M_1_PI + 1) * 0.5;
+  v = (theta * M_1_PI + 0.5);
 }
 
 Float AnimatedHitable::pdf_value(const point3f& o, const vec3f& v, random_gen& rng, Float time) {
@@ -53,7 +55,10 @@ bool AnimatedHitable::bounding_box(Float t0, Float t1, aabb& box) const {
   return(true);
 }
 
-bool AnimatedHitable::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng) {
+const bool AnimatedHitable::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng) const {
+  SCOPED_CONTEXT("Hit");
+  SCOPED_TIMER_COUNTER("Animation");
+  
   Transform InterpolatedPrimToWorld;
   PrimitiveToWorld.Interpolate(r.time(), &InterpolatedPrimToWorld);
   ray ray_interp = Inverse(InterpolatedPrimToWorld)(r);
@@ -68,7 +73,10 @@ bool AnimatedHitable::hit(const ray& r, Float t_min, Float t_max, hit_record& re
   return true;
 }
 
-bool AnimatedHitable::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sampler* sampler) {
+const bool AnimatedHitable::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sampler* sampler) const {
+  SCOPED_CONTEXT("Hit");
+  SCOPED_TIMER_COUNTER("Animation");
+  
   Transform InterpolatedPrimToWorld;
   PrimitiveToWorld.Interpolate(r.time(), &InterpolatedPrimToWorld);
   ray ray_interp = Inverse(InterpolatedPrimToWorld)(r);
