@@ -1,5 +1,6 @@
 #include "integrator.h"
 
+#include "RayMatrix.h"
 #include "float.h"
 // #define DEBUG
 #include "RcppThread.h"
@@ -10,17 +11,20 @@
 #include "filter.h"
 #include "sampler.h"
 #include "PreviewDisplay.h"
+#include "raylog.h"
 
 
 void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::size_t ns, int debug_channel,
                 Float min_variance, std::size_t min_adaptive_size, 
-                RayMatrix& routput, RayMatrix& goutput, RayMatrix& boutput, RayMatrix& alpha_output,
+                RayMatrix& rgb_output, RayMatrix& normalOutput, RayMatrix& albedoOutput, 
+                RayMatrix& alpha_output,
+                RayMatrix& draw_rgb_output,
                 bool progress_bar, int sample_method, Rcpp::NumericVector& stratified_dim,
                 bool verbose, RayCamera* cam, 
                 Float fov,
                 hitable_list& world, hitable_list& hlist,
                 Float clampval, std::size_t max_depth, std::size_t roulette_active,
-                PreviewDisplay& display) {
+                PreviewDisplay& display, IntegratorType integrator_type) {
   RProgress::RProgress pb_sampler("Generating Samples [:bar] :percent%");
   pb_sampler.set_width(70);
   RProgress::RProgress pb("Adaptive Raytracing [:bar] :percent%");
@@ -33,34 +37,38 @@ void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::si
     pb_sampler.set_total(ny);
     pb.set_total(ns);
   }
-#ifdef DEBUG
-  std::remove("rays.txt");
-#endif
-  RayMatrix routput2(nx,ny);
-  RayMatrix goutput2(nx,ny);
-  RayMatrix boutput2(nx,ny);
+  RayMatrix rgb_output2(nx,ny,3);
   display.write_fast_output = false;
+  bool adaptive_on = min_variance > 0;
   adaptive_sampler adaptive_pixel_sampler(numbercores, nx, ny, ns, debug_channel,
                                           min_variance, min_adaptive_size,
-                                          routput, goutput, boutput,
-                                          routput2, goutput2, boutput2,
-                                          alpha_output);
+                                          rgb_output, 
+                                          rgb_output2, 
+                                          normalOutput, albedoOutput,
+                                          alpha_output, draw_rgb_output,
+                                          adaptive_on);
   
   size_t nx_small = nx*0.25;
   size_t ny_small = ny*0.25;
-  RayMatrix routput_small(nx_small,ny_small);
-  RayMatrix goutput_small(nx_small,ny_small);
-  RayMatrix boutput_small(nx_small,ny_small);
-  RayMatrix routput_small2(nx_small,ny_small);
-  RayMatrix goutput_small2(nx_small,ny_small);
-  RayMatrix boutput_small2(nx_small,ny_small);
-  RayMatrix alpha_output_small(nx_small,ny_small);
+  RayMatrix rgb_output_small(nx_small,ny_small,3);
+  RayMatrix rgb_output_small2(nx_small,ny_small,3);
+  RayMatrix draw_rgb_output_small(nx_small,ny_small,3);
+
+  RayMatrix normal_output_small(nx_small,ny_small,3);
+  RayMatrix albedo_output_small(nx_small,ny_small,3);
+
+  RayMatrix alpha_output_small(nx_small,ny_small,1);
   
-  adaptive_sampler adaptive_pixel_sampler_small(numbercores, nx_small, ny_small, 1, debug_channel,
+  adaptive_sampler adaptive_pixel_sampler_small(numbercores, nx_small, ny_small, 
+                                                1, debug_channel,
                                                 0, 1,
-                                                routput_small, goutput_small, boutput_small,
-                                                routput_small2, goutput_small2, boutput_small2,
-                                                alpha_output_small);
+                                                rgb_output_small, 
+                                                rgb_output_small2, 
+                                                normal_output_small,
+                                                albedo_output_small,
+                                                alpha_output_small, 
+                                                draw_rgb_output_small, 
+                                                adaptive_on);
   std::vector<random_gen > rngs;
   std::vector<random_gen > rngs_small;
   
@@ -130,7 +138,8 @@ void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::si
                      nx, ny, s, sample_method,
                      &rngs, fov, &samplers,
                      cam, &world, &hlist,
-                     clampval, max_depth, roulette_active] (int k) {
+                     clampval, max_depth, roulette_active, 
+                     integrator_type] (int k) {
                        // MitchellFilter fil(vec2f(1.0),1./3.,1./3.);
                        int nx_begin = adaptive_pixel_sampler.pixel_chunks[k].startx;
                        int ny_begin = adaptive_pixel_sampler.pixel_chunks[k].starty;
@@ -148,7 +157,7 @@ void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::si
                            Float v = (Float(j) + u2.y()) / Float(ny);
                            
                            if(fov >= 0) {
-                             r = cam->get_ray(u,v, rand_to_unit(samplers[index]->Get2D()),
+                             r = cam->get_ray(u,v, convert_to_point3(rand_to_unit(samplers[index]->Get2D())),
                                               samplers[index]->Get1D());
                            } else {
                              CameraSample samp({1-u,1-v},samplers[index]->Get2D(), samplers[index]->Get1D());
@@ -156,9 +165,14 @@ void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::si
                            }
                            r.pri_stack = mat_stack;
                            bool alpha = false;
-                           point3f col = weight != 0 ? clamp_point(de_nan(color(r, &world, &hlist, max_depth, 
-                                                         roulette_active, rngs[index], samplers[index].get(), alpha)),
-                                               0, clampval) * weight * cam->get_iso() : 0;
+                           point3f color_sample;
+                           normal3f normal_sample;
+                           point3f albedo_sample;
+                           color(r, &world, &hlist, max_depth, roulette_active, rngs[index], 
+                                 samplers[index].get(), alpha,integrator_type,
+                                 color_sample, normal_sample, albedo_sample);
+                           point3f col = weight != 0 ? clamp_point(de_nan(color_sample), 
+                                                                   0, clampval) * weight * cam->get_iso() : 0;
                            if(alpha) {
                              adaptive_pixel_sampler.add_alpha_count(i,j);
                            }
@@ -168,11 +182,16 @@ void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::si
                            if(s % 2 == 0) {
                              adaptive_pixel_sampler.add_color_sec(i, j, col);
                            }
+                           adaptive_pixel_sampler.add_albedo(i, j, albedo_sample);
+                           adaptive_pixel_sampler.add_normal(i, j, normal_sample);
+
                            samplers[index]->StartNextSample();
                          }
                        }
-                       if((s % 2 == 1 && s > 3 && sample_method != 2) || (s % 2 == 1 && sample_method == 2 && s > 64)) {
-                         adaptive_pixel_sampler.test_for_convergence(k, s, nx_end, nx_begin, ny_end, ny_begin);
+                       if (adaptive_pixel_sampler.adaptive_on) {
+                        if((s % 2 == 1 && s > 3 && sample_method != 2) || (s % 2 == 1 && sample_method == 2 && s > 64)) {
+                          adaptive_pixel_sampler.test_for_convergence(k, s, nx_end, nx_begin, ny_end, ny_begin);
+                        }
                        }
                        delete mat_stack;
                      };
@@ -180,8 +199,10 @@ void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::si
         pool.push(worker, j);
       }
       pool.join();
-      if(s % 2 == 1 && s > 1) {
-        adaptive_pixel_sampler.split_remove_chunks(s);
+      if (adaptive_pixel_sampler.adaptive_on) {
+        if(s % 2 == 1 && s > 1) {
+          adaptive_pixel_sampler.split_remove_chunks(s);
+        }
       }
       adaptive_pixel_sampler.max_s++;
     } else {
@@ -189,7 +210,8 @@ void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::si
                      nx_small, ny_small, s, sample_method,
                      &rngs_small, fov, &samplers_small,
                      cam, &world, &hlist, 
-                     clampval, max_depth, roulette_active] (int k) {
+                     clampval, max_depth, roulette_active,
+                     integrator_type] (int k) {
                        // MitchellFilter fil(vec2f(1.0),1./3.,1./3.);
                        int nx_begin = adaptive_pixel_sampler_small.pixel_chunks[k].startx;
                        int ny_begin = adaptive_pixel_sampler_small.pixel_chunks[k].starty;
@@ -207,7 +229,7 @@ void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::si
                            Float v = (Float(j) + u2.y()) / Float(ny_small);
                            
                            if(fov >= 0) {
-                             r = cam->get_ray(u,v, rand_to_unit(samplers_small[index]->Get2D()),
+                             r = cam->get_ray(u,v, convert_to_point3(rand_to_unit(samplers_small[index]->Get2D())),
                                               samplers_small[index]->Get1D());
                            } else {
                              CameraSample samp({1-u,1-v},samplers_small[index]->Get2D(), samplers_small[index]->Get1D());
@@ -215,16 +237,21 @@ void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::si
                            }
                            r.pri_stack = mat_stack;
                            bool alpha = false;
-                           point3f col = weight != 0 ? clamp_point(de_nan(color(r, &world, &hlist, max_depth, 
-                                                                                roulette_active, 
-                                                                                rngs_small[index], 
-                                                                                samplers_small[index].get(),
-                                                                                alpha)
-                                                                          ),
+                           point3f color_sample;
+                           normal3f normal_sample;
+                           point3f albedo_sample;
+                           color(r, &world, &hlist, max_depth, roulette_active, 
+                                 rngs_small[index], 
+                                 samplers_small[index].get(),
+                                 alpha, integrator_type,
+                                 color_sample, normal_sample, albedo_sample);
+                           point3f col = weight != 0 ? clamp_point(de_nan(color_sample),
                                                                    0, clampval) * weight * cam->get_iso() : 0;
                            if(alpha) {
                              adaptive_pixel_sampler_small.add_alpha_count(i,j);
                            }
+                           adaptive_pixel_sampler_small.add_albedo(i, j, albedo_sample);
+                           adaptive_pixel_sampler_small.add_normal(i, j, normal_sample);
                            // col = col * fil.Evaluate(u2);
                            mat_stack->clear();
                            adaptive_pixel_sampler_small.add_color_main(i, j, col);
@@ -234,8 +261,10 @@ void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::si
                            samplers_small[index]->StartNextSample();
                          }
                        }
-                       if((s % 2 == 1 && s > 3 && sample_method != 2) || (s % 2 == 1 && sample_method == 2 && s > 64)) {
-                         adaptive_pixel_sampler_small.test_for_convergence(k, s, nx_end, nx_begin, ny_end, ny_begin);
+                       if (adaptive_pixel_sampler_small.adaptive_on) {
+                          if((s % 2 == 1 && s > 3 && sample_method != 2) || (s % 2 == 1 && sample_method == 2 && s > 64)) {
+                            adaptive_pixel_sampler_small.test_for_convergence(k, s, nx_end, nx_begin, ny_end, ny_begin);
+                          }
                        }
                        delete mat_stack;
                      };
@@ -244,9 +273,10 @@ void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::si
         pool.push(worker, j);
       }
       pool.join();
-      
-      if(s % 2 == 1 && s > 1) {
-        adaptive_pixel_sampler_small.split_remove_chunks(s);
+      if (adaptive_pixel_sampler_small.adaptive_on) {
+        if(s % 2 == 1 && s > 1) {
+          adaptive_pixel_sampler_small.split_remove_chunks(s);
+        }
       }
       adaptive_pixel_sampler_small.max_s++;
       
@@ -257,21 +287,35 @@ void pathtracer(std::size_t numbercores, std::size_t nx, std::size_t ny, std::si
         for(size_t jj = 0; jj < ny; jj++) {
           int iii = (Float)ii * ratio_x;
           int jjj = (Float)jj * ratio_y;
-          adaptive_pixel_sampler.r(ii,jj) = adaptive_pixel_sampler_small.r(iii,jjj);
-          adaptive_pixel_sampler.g(ii,jj) = adaptive_pixel_sampler_small.g(iii,jjj);
-          adaptive_pixel_sampler.b(ii,jj) = adaptive_pixel_sampler_small.b(iii,jjj);
-          adaptive_pixel_sampler.a(ii,jj) = adaptive_pixel_sampler_small.a(iii,jjj);
+          adaptive_pixel_sampler.rgb(ii,jj,0) = adaptive_pixel_sampler_small.rgb(iii,jjj,0);
+          adaptive_pixel_sampler.rgb(ii,jj,1) = adaptive_pixel_sampler_small.rgb(iii,jjj,1);
+          adaptive_pixel_sampler.rgb(ii,jj,2) = adaptive_pixel_sampler_small.rgb(iii,jjj,2);
+
+          adaptive_pixel_sampler.normalOutput(ii,jj,0) = adaptive_pixel_sampler_small.normalOutput(iii,jjj,0);
+          adaptive_pixel_sampler.normalOutput(ii,jj,1) = adaptive_pixel_sampler_small.normalOutput(iii,jjj,1);
+          adaptive_pixel_sampler.normalOutput(ii,jj,2) = adaptive_pixel_sampler_small.normalOutput(iii,jjj,2);
+
+          adaptive_pixel_sampler.albedoOutput(ii,jj,0) = adaptive_pixel_sampler_small.albedoOutput(iii,jjj,0);
+          adaptive_pixel_sampler.albedoOutput(ii,jj,1) = adaptive_pixel_sampler_small.albedoOutput(iii,jjj,1);
+          adaptive_pixel_sampler.albedoOutput(ii,jj,2) = adaptive_pixel_sampler_small.albedoOutput(iii,jjj,2);
+
+          adaptive_pixel_sampler.a(ii,jj,0) = adaptive_pixel_sampler_small.a(iii,jjj,0);
         }
       }
     }
     display.DrawImage(adaptive_pixel_sampler, adaptive_pixel_sampler_small,
                       s, pb, progress_bar,
                       (Float)s/(Float)ns, &world, rng_interactive);
-    if(display.terminate) {
+    if(display.terminate && display.preview) {
       adaptive_pixel_sampler.ns = s;
       adaptive_pixel_sampler.max_s = s;
       break;
     }
   }
+
+  #ifdef RAY_COLOR_DEBUG
+  Rcpp::Rcout << "ns: " << adaptive_pixel_sampler.ns << " adaptive_pixel_sampler.rgb pre divide:\n";
+  adaptive_pixel_sampler.rgb.print();
+  #endif 
   adaptive_pixel_sampler.write_final_pixels();
 }
