@@ -1,15 +1,24 @@
 #include "sphere.h"
+#include "raylog.h"
+#include "vectypes.h"
 
 // #include "RcppThread.h"
 
-bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng) {
+const bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng) const {
+  SCOPED_CONTEXT("Hit");
+  SCOPED_TIMER_COUNTER("Sphere");
   vec3f oErr, dErr;
   ray r2 = (*WorldToObject)(r, &oErr, &dErr);
   // Compute quadratic sphere coefficients
   
   // Initialize _EFloat_ ray coordinate values
-  EFloat ox(r2.origin().x(), oErr.x()), oy(r2.origin().y(), oErr.y()), oz(r2.origin().z(), oErr.z());
-  EFloat dx(r2.direction().x(), dErr.x()), dy(r2.direction().y(), dErr.y()), dz(r2.direction().z(), dErr.z());
+  EFloat ox(r2.origin().x(), oErr.x()), 
+         oy(r2.origin().y(), oErr.y()), 
+         oz(r2.origin().z(), oErr.z());
+  EFloat dx(r2.direction().x(), dErr.x()), 
+         dy(r2.direction().y(), dErr.y()), 
+         dz(r2.direction().z(), dErr.z());
+         
   EFloat a = dx * dx + dy * dy + dz * dz;
   EFloat b = 2 * (dx * ox + dy * oy + dz * oz);
   EFloat c = ox * ox + oy * oy + oz * oz - EFloat(radius) * EFloat(radius);
@@ -29,7 +38,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random
     if(temp1 < t_max && temp1 > t_min) {
       point3f p1 = r2.point_at_parameter((Float)temp1);
       p1 *= radius / p1.length(); 
-      vec3f normal = p1 / radius;
+      vec3f normal = convert_to_vec3(p1 / radius);
       get_sphere_uv(normal, u, v);
       if(alpha_mask->value(u, v, rec.p) < rng.unif_rand()) {
         is_hit = false;
@@ -38,7 +47,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random
     if(temp2 < t_max && temp2 > t_min) {
       point3f p2 = r2.point_at_parameter((Float)temp2);
       p2 *= radius / p2.length(); 
-      vec3f normal = p2 / radius;
+      vec3f normal = convert_to_vec3(p2 / radius);
       get_sphere_uv(normal, u, v);
       if(alpha_mask->value(u, v, rec.p) < rng.unif_rand()) {
         if(!is_hit) {
@@ -52,7 +61,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random
     rec.t = (Float)temp1;
     rec.p = r2.point_at_parameter(rec.t);
     rec.p *= radius / rec.p.length(); 
-    rec.normal = vec3f(rec.p) / radius;
+    rec.normal = convert_to_normal3(rec.p) / radius;
     
     //Interaction information
     Float zRadius = std::sqrt(rec.p.x() * rec.p.x()  + rec.p.z()  * rec.p.z() );
@@ -60,17 +69,17 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random
     Float cosPhi = rec.p.x() * invZRadius;
     Float sinPhi = rec.p.z() * invZRadius;
     Float theta = std::acos(clamp(rec.p.z() / radius, -1, 1));
-    rec.dpdu = 2 * M_PI * vec3f(-rec.p.z(), 0, rec.p.x());
-    rec.dpdv = 2 * M_PI * vec3f(rec.p.z() * cosPhi, rec.p.z() * sinPhi, -radius * std::sin(theta));
+    rec.dpdu = 2 * static_cast<Float>(M_PI) * vec3f(-rec.p.z(), 0, rec.p.x());
+    rec.dpdv = 2 * static_cast<Float>(M_PI) * vec3f(rec.p.z() * cosPhi, rec.p.z() * sinPhi, -radius * std::sin(theta));
     get_sphere_uv(rec.normal, rec.u, rec.v);
     rec.has_bump = bump_tex ? true : false;
     
     if(bump_tex) {
       point3f bvbu = bump_tex->value(rec.u,rec.v, rec.p);
-      rec.bump_normal = rec.normal + normal3f(bvbu.x() * rec.dpdu + bvbu.y() * rec.dpdv); 
+      rec.bump_normal = rec.normal + convert_to_normal3(bvbu.x() * rec.dpdu + bvbu.y() * rec.dpdv); 
       rec.bump_normal.make_unit_vector();
     }
-    rec.pError = gamma(5) * Abs(rec.p);
+    rec.pError = convert_to_vec3(gamma(5) * Abs(rec.p));
     rec = (*ObjectToWorld)(rec);
     rec.normal *= reverseOrientation  ? -1 : 1;
     rec.bump_normal *= reverseOrientation  ? -1 : 1;
@@ -85,7 +94,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random
     rec.t = (Float)temp2;
     rec.p = r2.point_at_parameter(rec.t);
     rec.p *= radius / rec.p.length();
-    rec.normal = vec3f(rec.p) / radius;
+    rec.normal = convert_to_normal3(rec.p) / radius;
     
     //Interaction information
     Float zRadius = std::sqrt(rec.p.x() * rec.p.x()  + rec.p.z()  * rec.p.z() );
@@ -93,14 +102,14 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random
     Float cosPhi = rec.p.x() * invZRadius;
     Float sinPhi = rec.p.z() * invZRadius;
     Float theta = std::acos(clamp(rec.p.z() / radius, -1, 1));
-    rec.dpdu = 2 * M_PI * vec3f(-rec.p.z(), 0, rec.p.x());
-    rec.dpdv = 2 * M_PI * vec3f(rec.p.z() * cosPhi, rec.p.z() * sinPhi, -radius * std::sin(theta));
+    rec.dpdu = 2 * static_cast<Float>(M_PI) * vec3f(-rec.p.z(), 0, rec.p.x());
+    rec.dpdv = 2 * static_cast<Float>(M_PI) * vec3f(rec.p.z() * cosPhi, rec.p.z() * sinPhi, -radius * std::sin(theta));
     get_sphere_uv(rec.normal, rec.u, rec.v);
     rec.has_bump = bump_tex ? true : false;
     
     if(bump_tex) {
       point3f bvbu = bump_tex->value(rec.u,rec.v, rec.p);
-      rec.bump_normal = rec.normal +  normal3f(bvbu.x() * rec.dpdu + bvbu.y() * rec.dpdv); 
+      rec.bump_normal = rec.normal +  convert_to_normal3(bvbu.x() * rec.dpdu + bvbu.y() * rec.dpdv); 
       rec.bump_normal.make_unit_vector();
     }
     
@@ -108,7 +117,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random
       rec.normal = -rec.normal;
       rec.bump_normal = -rec.bump_normal;
     }
-    rec.pError = gamma(5) * Abs(rec.p);
+    rec.pError = convert_to_vec3(gamma(5) * Abs(rec.p));
     rec = (*ObjectToWorld)(rec);
     rec.normal *= reverseOrientation  ? -1 : 1;
     rec.bump_normal *= reverseOrientation  ? -1 : 1;
@@ -124,7 +133,10 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random
 }
 
 
-bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sampler* sampler) {
+const bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sampler* sampler) const {
+  SCOPED_CONTEXT("Hit");
+  SCOPED_TIMER_COUNTER("Sphere");
+  
   vec3f oErr, dErr;
   ray r2 = (*WorldToObject)(r, &oErr, &dErr);
   // Compute quadratic sphere coefficients
@@ -150,7 +162,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sample
     if(temp1 < t_max && temp1 > t_min) {
       point3f p1 = r2.point_at_parameter((Float)temp1);
       p1 *= radius / p1.length(); 
-      vec3f normal = p1 / radius;
+      vec3f normal = convert_to_vec3(p1) / radius;
       get_sphere_uv(normal, u, v);
       if(alpha_mask->value(u, v, rec.p) < sampler->Get1D()) {
         is_hit = false;
@@ -159,7 +171,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sample
     if(temp2 < t_max && temp2 > t_min) {
       point3f p2 = r2.point_at_parameter((Float)temp2);
       p2 *= radius / p2.length(); 
-      vec3f normal = p2 / radius;
+      vec3f normal = convert_to_vec3(p2) / radius;
       get_sphere_uv(normal, u, v);
       if(alpha_mask->value(u, v, rec.p) < sampler->Get1D()) {
         if(!is_hit) {
@@ -173,7 +185,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sample
     rec.t = (Float)temp1;
     rec.p = r2.point_at_parameter(rec.t);
     rec.p *= radius / rec.p.length(); 
-    rec.normal = vec3f(rec.p) / radius;
+    rec.normal = convert_to_normal3(rec.p) / radius;
     
     //Interaction information
     Float zRadius = std::sqrt(rec.p.x() * rec.p.x()  + rec.p.z()  * rec.p.z() );
@@ -181,17 +193,17 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sample
     Float cosPhi = rec.p.x() * invZRadius;
     Float sinPhi = rec.p.z() * invZRadius;
     Float theta = std::acos(clamp(rec.p.z() / radius, -1, 1));
-    rec.dpdu = 2 * M_PI * vec3f(-rec.p.z(), 0, rec.p.x());
-    rec.dpdv = 2 * M_PI * vec3f(rec.p.z() * cosPhi, rec.p.z() * sinPhi, -radius * std::sin(theta));
+    rec.dpdu = 2 * static_cast<Float>(M_PI) * vec3f(-rec.p.z(), 0, rec.p.x());
+    rec.dpdv = 2 * static_cast<Float>(M_PI) * vec3f(rec.p.z() * cosPhi, rec.p.z() * sinPhi, -radius * std::sin(theta));
     get_sphere_uv(rec.normal, rec.u, rec.v);
     rec.has_bump = bump_tex ? true : false;
     
     if(bump_tex) {
       point3f bvbu = bump_tex->value(rec.u,rec.v, rec.p);
-      rec.bump_normal = rec.normal + normal3f(bvbu.x() * rec.dpdu + bvbu.y() * rec.dpdv); 
+      rec.bump_normal = rec.normal + convert_to_normal3(bvbu.x() * rec.dpdu + bvbu.y() * rec.dpdv); 
       rec.bump_normal.make_unit_vector();
     }
-    rec.pError = gamma(5) * Abs(rec.p);
+    rec.pError = convert_to_vec3(gamma(5) * Abs(rec.p));
     rec = (*ObjectToWorld)(rec);
     rec.normal *= reverseOrientation  ? -1 : 1;
     rec.bump_normal *= reverseOrientation  ? -1 : 1;
@@ -207,7 +219,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sample
     rec.t = (Float)temp2;
     rec.p = r2.point_at_parameter(rec.t);
     rec.p *= radius / rec.p.length();
-    rec.normal = vec3f(rec.p) / radius;
+    rec.normal = convert_to_normal3(rec.p) / radius;
     
     //Interaction information
     Float zRadius = std::sqrt(rec.p.x() * rec.p.x()  + rec.p.z()  * rec.p.z() );
@@ -215,14 +227,14 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sample
     Float cosPhi = rec.p.x() * invZRadius;
     Float sinPhi = rec.p.z() * invZRadius;
     Float theta = std::acos(clamp(rec.p.z() / radius, -1, 1));
-    rec.dpdu = 2 * M_PI * vec3f(-rec.p.z(), 0, rec.p.x());
-    rec.dpdv = 2 * M_PI * vec3f(rec.p.z() * cosPhi, rec.p.z() * sinPhi, -radius * std::sin(theta));
+    rec.dpdu = 2 * static_cast<Float>(M_PI) * vec3f(-rec.p.z(), 0, rec.p.x());
+    rec.dpdv = 2 * static_cast<Float>(M_PI) * vec3f(rec.p.z() * cosPhi, rec.p.z() * sinPhi, -radius * std::sin(theta));
     get_sphere_uv(rec.normal, rec.u, rec.v);
     rec.has_bump = bump_tex ? true : false;
     
     if(bump_tex) {
       point3f bvbu = bump_tex->value(rec.u,rec.v, rec.p);
-      rec.bump_normal = rec.normal +  normal3f(bvbu.x() * rec.dpdu + bvbu.y() * rec.dpdv); 
+      rec.bump_normal = rec.normal +  convert_to_normal3(bvbu.x() * rec.dpdu + bvbu.y() * rec.dpdv); 
       rec.bump_normal.make_unit_vector();
     }
     
@@ -230,7 +242,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sample
       rec.normal = -rec.normal;
       rec.bump_normal = -rec.bump_normal;
     }
-    rec.pError = gamma(5) * Abs(rec.p);
+    rec.pError = convert_to_vec3(gamma(5) * Abs(rec.p));
     rec = (*ObjectToWorld)(rec);
     rec.normal *= reverseOrientation  ? -1 : 1;
     rec.bump_normal *= reverseOrientation  ? -1 : 1;
@@ -245,10 +257,75 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sample
   return(false);
 }
 
+bool sphere::HitP(const ray& r, Float t_min, Float t_max, random_gen& rng) const {
+  SCOPED_CONTEXT("Hit");
+  SCOPED_TIMER_COUNTER("Sphere");
+  vec3f oErr, dErr;
+  ray r2 = (*WorldToObject)(r, &oErr, &dErr);
+  // Compute quadratic sphere coefficients
+  
+  // Initialize _EFloat_ ray coordinate values
+  EFloat ox(r2.origin().x(), oErr.x()), 
+         oy(r2.origin().y(), oErr.y()), 
+         oz(r2.origin().z(), oErr.z());
+  EFloat dx(r2.direction().x(), dErr.x()), 
+         dy(r2.direction().y(), dErr.y()), 
+         dz(r2.direction().z(), dErr.z());
+         
+  EFloat a = dx * dx + dy * dy + dz * dz;
+  EFloat b = 2 * (dx * ox + dy * oy + dz * oz);
+  EFloat c = ox * ox + oy * oy + oz * oz - EFloat(radius) * EFloat(radius);
+  
+  // Solve quadratic equation for _t_ values
+  EFloat temp1, temp2;
+  if (!Quadratic(a, b, c, &temp1, &temp2)) {
+    return(false);
+  }
+  if(temp1 < t_max && temp1 > t_min) {
+    return(true);
+  }
+  if(temp2 < t_max && temp2 > t_min) {
+    return(true);
+  }
+  return(false);
+}
+
+bool sphere::HitP(const ray& r, Float t_min, Float t_max, Sampler* sampler) const {
+  SCOPED_CONTEXT("Hit");
+  SCOPED_TIMER_COUNTER("Sphere");
+  vec3f oErr, dErr;
+  ray r2 = (*WorldToObject)(r, &oErr, &dErr);
+  // Compute quadratic sphere coefficients
+  
+  // Initialize _EFloat_ ray coordinate values
+  EFloat ox(r2.origin().x(), oErr.x()), 
+         oy(r2.origin().y(), oErr.y()), 
+         oz(r2.origin().z(), oErr.z());
+  EFloat dx(r2.direction().x(), dErr.x()), 
+         dy(r2.direction().y(), dErr.y()), 
+         dz(r2.direction().z(), dErr.z());
+         
+  EFloat a = dx * dx + dy * dy + dz * dz;
+  EFloat b = 2 * (dx * ox + dy * oy + dz * oz);
+  EFloat c = ox * ox + oy * oy + oz * oz - EFloat(radius) * EFloat(radius);
+  
+  // Solve quadratic equation for _t_ values
+  EFloat temp1, temp2;
+  if (!Quadratic(a, b, c, &temp1, &temp2)) {
+    return(false);
+  }
+  if(temp1 < t_max && temp1 > t_min) {
+    return(true);
+  }
+  if(temp2 < t_max && temp2 > t_min) {
+    return(true);
+  }
+  return(false);
+}
+
 
 Float sphere::pdf_value(const point3f& o, const vec3f& v, random_gen& rng, Float time) {
-  hit_record rec;
-  if(!this->hit(ray(o,v), 0.001, FLT_MAX, rec, rng)) {
+  if(!this->HitP(ray(o,v), 0.001, FLT_MAX, rng)) {
     return(0);
   }
   point3f pCenter = (*ObjectToWorld)(point3f(0.f, 0.f, 0.f));
@@ -266,8 +343,7 @@ Float sphere::pdf_value(const point3f& o, const vec3f& v, random_gen& rng, Float
 
 
 Float sphere::pdf_value(const point3f& o, const vec3f& v, Sampler* sampler, Float time) {
-  hit_record rec;
-  if(!this->hit(ray(o,v), 0.001, FLT_MAX, rec, sampler)) {
+  if(!this->HitP(ray(o,v), 0.001, FLT_MAX, sampler)) {
     return(0);
   }
   point3f pCenter = (*ObjectToWorld)(point3f(0.f, 0.f, 0.f));
@@ -307,14 +383,14 @@ vec3f sphere::random(const point3f& o, random_gen& rng, Float time) {
     /* Fall back to a Taylor series expansion for small angles, where
      the standard approach suffers from severe cancellation errors */
     sinTheta2 = sinThetaMax2 * u[0];
-    cosTheta = std::sqrt(1 - sinTheta2);
+    cosTheta = std::sqrt(static_cast<Float>(1) - sinTheta2);
   }
   
   // Compute angle $\alpha$ from center of sphere to sampled point on surface
   Float cosAlpha = sinTheta2 * invSinThetaMax +
     cosTheta * std::sqrt(std::fmax((Float)0.f, 1.f - sinTheta2 * invSinThetaMax * invSinThetaMax));
   Float sinAlpha = std::sqrt(std::fmax((Float)0.f, 1.f - cosAlpha*cosAlpha));
-  Float phi = u.e[1] * 2 * M_PI;
+  Float phi = u.e[1] * 2 * static_cast<Float>(M_PI);
   
   // Compute surface normal and sampled point on sphere
   vec3f nWorld = SphericalDirection(sinAlpha, cosAlpha, phi, -uvw.u(), -uvw.v(), -uvw.w());
@@ -353,12 +429,12 @@ vec3f sphere::random(const point3f& o, Sampler* sampler, Float time) {
   Float cosAlpha = sinTheta2 * invSinThetaMax +
     cosTheta * std::sqrt(std::fmax((Float)0.f, 1.f - sinTheta2 * invSinThetaMax * invSinThetaMax));
   Float sinAlpha = std::sqrt(std::fmax((Float)0.f, 1.f - cosAlpha*cosAlpha));
-  Float phi = u.e[1] * 2 * M_PI;
+  Float phi = u.e[1] * 2 * static_cast<Float>(M_PI);
   
   // Compute surface normal and sampled point on sphere
   vec3f nWorld = SphericalDirection(sinAlpha, cosAlpha, phi, -uvw.u(), -uvw.v(), -uvw.w());
   point3f pWorld = pCenter + radius * point3f(nWorld.x(), nWorld.y(), nWorld.z());
-  return unit_vector(pWorld-o);
+  return unit_vector(pWorld-o); 
 }
 
 bool sphere::bounding_box(Float t0, Float t1, aabb& box) const {
